#!/usr/bin/env python

"""
This is a small script which converts Pegasus Mail folders (.pmm) to
unix mailboxes. Run it on several .ppm files and it will create their
unix version.

The pegasusmail2unixmailbox.py package is released to the public domain.
You can do whatever you want with it, and you are responsible for
yourself. Basically, imagine this simply appeared on your hard disk and
don't make questions.

Written by Grzegorz Adam Hankiewicz. Possible contact emails:
gradha@terra.es, gradha@users.sourceforge.net.
Download this and other things from http://gradha.infierno.org/.
"""

import string, sys

# Should be enough to process most emails without problem
MAX_TEMPORARY_MEMORY = 1024 * 1024
version = "1.0 pegasusmail2unixmailbox.py"


def is_character(char):
   """Checks if char is in the string.letters group"""
   if char in string.letters: return 1
   return 0



def write_mail(file, buffer, start, end):
   """Write a chunk of mail from the buffer to the opened file.

   The buffer will usually contain the whole pegasus mailbox string, so you
   need the start and end of the mail being written. Of course it has to
   translate all '\r' and chr(26) to nothing, and add a fake From line
   before each mail.

   Since mail chunks can be really big, this is implemented in two algoritms:
   the brute force algorithm is used when the size of the mail being written
   is bigger than MAX_TEMPORARY_MEMORY. This is a C-like write char by char
   slow algorithm. Otherwise, a pythonic is used.

   The latter consumes more memory but is faster. Tune MAX_TEMPORARY_MEMORY
   to something your computer can manage.
   """
   if start >= end: return
   file.write("From jander@mailer.com Mon Aug 16 23:59:25 1999\n")
   if end - start < MAX_TEMPORARY_MEMORY:
      file.write(string.replace(string.replace(buffer[start:end], '\r', ""),
         chr(26), ""))
   else:
      while start < end:
         if not(buffer[start] == '\r' or buffer[start] == chr(26)):
            file.write(buffer[start])
         start = start + 1



def process_pmm(pmm_filename):
   """Entry point for each pegasus folder processing.

   This will open the pegasus mail folder, read the WHOLE thing to memory,
   and process it. This means that you need so much free RAM like your
   biggest .ppm file plus a few megabytes for python and temporary mem.
   Too lazy to implement a less resource hungry algorithm...
   """
   file = open(pmm_filename, "rb")
   buf = file.read()
   file.close()

   # Get the name of the 'real' mailbox
   end_title = string.find(buf, chr(0))
   mbox_filename = "%s.unixmbox" % string.strip(buf[:end_title])
   try:
      file = open(mbox_filename, "wt")
   except IOError:
      # Hmm... try again filtering weird ascii characters
      mbox_filename = "%s.unixmbox" % filter(is_character, mbox_filename[:-9])
      file = open(mbox_filename, "wt")
   print "%s -> %s" % (pmm_filename, mbox_filename)

   # Find beginning of first received mail
   start = string.find(buf, "%sReceived: " % chr(0))
   start = start + 1
   mails = 0
   while 1: # infinite loop processing mails one by one
      end = string.find(buf, chr(26), start)
      if end <= start: break
      write_mail(file, buf, start, end)
      start = end + 1
      mails = mails + 1

   file.close()
   print "%d mails converted." % mails

   

def show_help():
   """Show commandline help."""
   print "pegasusmail2unixmailbox written by Grzegorz Adam Hankiewicz."
   print "This is public domain, do what you want with it and don't blame me."
   print
   print "Usage: pegasusmail2unixmailbox.py [-h -v] .pmm files"
   print
   print " -h       Show this help and exit"
   print " -v       Show version number and exit"

   

if __name__ == "__main__":   
   """Entry point for userland."""
   params = sys.argv[1:]
   if not len(params) or "-h" in params:
      show_help()
      sys.exit(0)
      
   if "-v" in params:
      print version
      sys.exit(0)
      
   for file in params: process_pmm(file)
